import logging
# Django.
from django import forms
from django.utils.translation import ugettext_lazy as _

# Local Django.
from .models import User
from . import constants

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger('EmbelezaMais')


class RegisterForm(forms.ModelForm):
    # Form Fields.
    name = forms.CharField(label='NAME',
                           max_length=60)

    email = forms.EmailField(label='EMAIL')

    class Meta:
        model = User
        fields = [
            'name',
            'email'
        ]

    # Front-end validation function for client register page.
    def clean(self, *args, **kwargs):
        email = self.cleaned_data.get('email')
        password = self.cleaned_data.get('password')
        password_confirmation = self.cleaned_data.get('password_confirmation')

        email_from_database = User.objects.filter(email=email)

        if email_from_database.exists():
            raise forms.ValidationError(('Email already registered'))
        elif len(password) < 8:
            raise forms.ValidationError(
                ('Password must be between 8 and 12 characters'))
        elif len(password) > 12:
            raise forms.ValidationError(
                ('Password must be between 8 and 12 characters'))
        elif password != password_confirmation:
            raise forms.ValidationError(('Passwords do not match.'))

        return super(RegisterForm, self).clean(*args, **kwargs)


class LoginForm(forms.Form):
    email = forms.EmailField()
    password = forms.CharField(
        widget=forms.PasswordInput, label=_(constants.PASSWORD))

    def clean(self, *args, **kwargs):
        password = self.cleaned_data.get("password")

        if len(password) < constants.PASSWORD_MIN_LENGTH:
            raise forms.ValidationError(
                {'password': [_(constants.PASSWORD_SIZE)]})
        elif len(password) > constants.PASSWORD_MAX_LENGTH:
            raise forms.ValidationError(
                {'password': [_(constants.PASSWORD_SIZE)]})
        else:
            pass

        return super(LoginForm, self).clean(*args, **kwargs)

    class Meta:
        model = User



# standard library
import abc

# Django
from django.shortcuts import render

# local django
from provider.models import Provider
from service.models import Service
from search.forms import SearchForm


class Search:
    @abc.abstractmethod
    def to_search(self):
        return

    @abc.abstractmethod
    def get_list_itens(self):
        return

    @abc.abstractmethod
    def set_list_itens(self, list_itens):
        return


class SearchName(Search):
    def __init__(self, name, type_search="provider", **kwargs):
         self.name = name
         self.type_search = type_search
         self.list_itens = None

    def to_search(self):
        if self.type_search is 'provider':
            self.list_itens = Provider.objects.filter(name__icontains=self.name)
        elif self.type_search is 'service':
            self.list_itens = Service.objects.filter(name__icontains=self.name)
            pass

    def get_list_itens(self):
        return self.list_itens

    def set_list_itens(self, list_itens):
        self.list_itens = list_itens


class SearchDecorator(Search):
    def __init__(self, search, **kwargs):
         self.search = search

    def get_list_itens(self):
        return self.search.get_list_itens()

    def set_list_itens(self, list_itens):
        self.search.set_list_itens(list_itens)

    def to_search(self):
        # Search base or higher priority
        self.search.to_search()

        # Refine search using decorator
        self.refine_search()

    @abc.abstractmethod
    def refine_search(self):
        """
        Use list_itens as a basis for refinement.
        get_list_itens and set_list_itens
        """
        return


class ProviderSearchAndList(ListView):
    model = Provider
    template_name = None
    context_object_name = 'providers'
    paginate_by = 10
    type_search='provider'
    name = ''

    def get_queryset(self):
        search = self.define_researches()
        search.to_search()
        return search.get_list_itens()

    def get(self, request, *args, **kwargs):
        form = SearchForm(request.GET or None)
        return render(request, self.template_name, {'providers': self.get_queryset(), 'form': form})

    def post(self, request, *args, **kwargs):
        form = SearchForm(request.POST or None)
        if form.is_valid():
            self.get_fields(form)

        return render(request, self.template_name, {'providers': self.get_queryset(), 'form': form})

    def get_fields(self, form):
        self.name = form.cleaned_data.get('name')

    def define_researches(self):
        search = SearchName(name=self.name, type_search=self.type_search)
        return search



from django.db import models


class Service(models.Model):
    name = models.CharField(max_length=120)
    description = models.TextField(max_length=500)
    price = models.FloatField(blank=False, null=False)

    def __str__(self):
        return self.name


class Combo(Service):
    specification = models.CharField(max_length=120)
    services = models.ManyToManyField(Service, through='ServiceCombo', related_name='services')

    def add(self, service):
        ServicesCombo.objects.create(service_combo=self, service_in_combo=service)

    def remove(self, service):
        service_combo = ServicesCombo.objects.filter(service_combo=self, service_in_combo=service)
        service_combo.delete()


class ServiceCombo(models.Model):
    service_combo = models.ForeignKey(Combo, related_name='service_combo')
    service_in_combo = models.ForeignKey(Service, related_name='service_in_combo')



import datetime

# Django.
from django.db import models
from django.contrib.auth.models import AbstractBaseUser, BaseUserManager, PermissionsMixin
from django.db.models import EmailField
from django.core import validators
from . import constants


class UserManager(BaseUserManager):
    def create_user(self, email, password, name, **kwargs):

        if not email:
            raise ValueError('The given email must be set')

        email = self.normalize_email(email)
        user = self.model(email=email,
                          name=name,
                          is_active=False,
                          **kwargs)
        user.set_password(password)
        user.save(using=self.db)

        return user

    def create_superuser(self, email, password, name, **kwargs):

        email = self.normalize_email(email)
        user = self.model(email=email,
                          name=name,
                          is_staff=True,
                          is_active=True,
                          is_superuser=True,
                          **kwargs)

        user.set_password(password)
        user.save(using=self.db)

        return user


class Email(EmailField):
    validator = validators.EmailValidator(message=constants.EMAIL_FORMAT)

    default_validators = [validator]

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = constants.EMAIL_FIELD_LENGTH
        kwargs['default'] = ''
        kwargs['unique'] = True
        super(EmailField, self).__init__(*args, **kwargs)


class Name(models.CharField):
    validator_max_length = validators.MaxLengthValidator(constants.NAME_FIELD_LENGTH,
                                                         message=constants.NAME_SIZE)
    validator_format = validators.RegexValidator(regex=r'^[a-zA-Zá-úÀ-Úã-õ-Ã-Õ ]+$',
                                                 message=constants.NAME_CHARACTERS)

    default_validators = [validator_max_length, validator_format]

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 60
        kwargs['default'] = ''
        super(models.CharField, self).__init__(*args, **kwargs)


class User(AbstractBaseUser, PermissionsMixin):
    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = ['name']

    email = Email()
    name = Name()
    is_active = models.BooleanField(default=False)
    is_staff = models.BooleanField(default=False)
    objects = UserManager()

    def get_full_name(self):
        return self.email

    def get_short_name(self):
        return self.email


class UserProfile(models.Model):
    user = models.OneToOneField(User)
    activation_key = models.CharField(max_length=40, blank=True)
    key_expires = models.DateTimeField(default=datetime.date.today())

    def __str__(self):
        return self.user.email

    class Meta:
        verbose_name_plural = u'Perfil de Usuario'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models


# Create your models here.
class Provider(models.Model):
    name = models.CharField(max_length=120)
    branch = models.CharField(max_length=120)
    description = models.CharField(max_length=120)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-24 02:57
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Service',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=120)),
                ('description', models.TextField(max_length=500)),
                ('price', models.FloatField()),
            ],
        ),
        migrations.CreateModel(
            name='ServiceCombo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='Combo',
            fields=[
                ('service_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='service.Service')),
                ('specification', models.CharField(max_length=120)),
            ],
            bases=('service.service',),
        ),
        migrations.AddField(
            model_name='servicecombo',
            name='service_in_combo',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='service_in_combo', to='service.Service'),
        ),
        migrations.AddField(
            model_name='servicecombo',
            name='service_combo',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='service_combo', to='service.Combo'),
        ),
        migrations.AddField(
            model_name='combo',
            name='services',
            field=models.ManyToManyField(related_name='services', through='service.ServiceCombo', to='service.Service'),
        ),
    ]



# EMAIL FIELDS.
EMAIL_FIELD_LENGTH = 60

# NAME FIELDS.
NAME_FIELD_LENGTH = 80

# EMAIL VALIDATION MESSAGES.
EMAIL_FORMAT = 'Enter a valid email address'

# NAME VALIDATION MESSAGES
NAME_SIZE = 'Your name exceeds 60 characteres'
NAME_CHARACTERS = 'Your name can\'t have special characters'

# PASSWORD FIELDS
PASSWORD = 'Password'
PASSWORD_MIN_LENGTH = 8
PASSWORD_MAX_LENGTH = 12

# PASSWORD VALIDATION MESSAGES.
PASSWORD_SIZE = 'Password must be between 8 and 12 characters'



#!/usr/bin/env python
import os
import sys

if __name__ == "__main__":
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "ssmais.settings")
    try:
        from django.core.management import execute_from_command_line
    except ImportError:
        # The above import may fail for some other reason. Ensure that the
        # issue is really that Django is missing to avoid masking other
        # exceptions on Python 2.
        try:
            import django
        except ImportError:
            raise ImportError(
                "Couldn't import Django. Are you sure it's installed and "
                "available on your PYTHONPATH environment variable? Did you "
                "forget to activate a virtual environment?"
            )
        raise
    execute_from_command_line(sys.argv)



from pip.req import parse_requirements
import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

# parse_requirements() returns generator of pip.req.InstallRequirement objects
install_reqs = parse_requirements('requirements.txt', session='hack')

# reqs is a list of requirement
requires = [str(ir.req) for ir in install_reqs]


setup(
    name='ssmais',
    version='0.7.3',
    include_package_data=True,
    packages=find_packages(),
    license='GPLv3',
    description='SSMais is a Django application that aims to perform reservation management of a service over the Web.',
    long_description='''SSMais is a Django application that aims to perform reservation management of a service over the Web.
    The main purpose of this application is to allow the reuse of features that are generally used in scheduling
    software such as scheduling, searching and registering services.
    ''',
    url='https://github.com/Software-Design-2017/SSMais',
    author='Mais Software Inc.',
    author_email=' maissoftwareinc@gmail.com',
    install_requires=requires,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    platforms='any',
)



from django import forms

from .models import (
    Service, Combo, ServiceCombo
)


class ServiceForm(forms.ModelForm):
    provider_create = None

    def __init__(self, **kw):
        super(ServiceForm, self).__init__(**kw)
        self.fields['service_in_combo'].queryset = Service.objects.filter(provider=self.provider_create,
                                                                          combo__isnull=True)
        for field_name, field in self.fields.items():
            field.widget.attrs['class'] = 'form-control'

    @staticmethod
    def __create_service_form__(provider):
        ServiceForm.provider_create = provider
        return ServiceForm

    class Meta:
        model = ServiceCombo
        fields = ['service_in_combo']


class ComboForm():
    provider = None
    ServiceComboFormSet = forms.inlineformset_factory(Combo, ServiceCombo,
                                                      form=ServiceForm, extra=0,
                                                      fk_name='service_combo',
                                                      min_num=2, validate_min=True)

    def __init__(self, provider):
        self.ServiceComboFormSet.form = ServiceForm.__create_service_form__(provider=provider)


class BaseServiceForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(FormService, self).__init__(*args, **kwargs)
        for field_name, field in self.fields.items():
            field.widget.attrs['class'] = 'form-control'

    class Meta:
        model = Service
        exclude = ['provider']
